/******************************************************************************
 *
 *       XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
 *       AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
 *       SOLUTIONS FOR XILINX DEVICES. BY PROVIDING THIS DESIGN, CODE,
 *       OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
 *       APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
 *       THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
 *       AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
 *       FOR YOUR IMPLEMENTATION. XILINX EXPRESSLY DISCLAIMS ANY
 *       WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
 *       IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
 *       REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
 *       INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *       FOR A PARTICULAR PURPOSE.
 *
 *	 Copyright (C) 2006 Vreelin Engineering, Inc.  All Rights Reserved.
 *       (c) Copyright 2006-2007 Xilinx Inc.
 *       All rights reserved.
 *
 *****************************************************************************/
/*****************************************************************************/
/**
*
* @file xusb.h
*
* This file contains the implementation of the XUsb component. It is the
* driver for the USB device controller.
*
* The USB device controller supports the following features:
*	- USB 2.0 Specification supporting High/Full/Low Speed
*	- 8 Endpoints
*		- 1 Control Endpoint
*		- 7 Configurable Endpoints, which can be configured
*			as IN or OUT , and configurable as Interrupt or Bulk or
* 			Isochronous
*	- 2 Ping Pong Buffers for all the endpoints except the Control Endpoint
*
* <b>Initialization & Configuration</b>
*
* The device driver enables higher layer software (e.g., an application) to
* communicate to the USB device. Apart from transmission and reception of
* USB frames the driver also handles the configuration of the device. A single
* device driver can support multiple USB devices.
*
* {XUsb_CfgInitialize()} API is used to initialize the EPB CAN device.
* The user needs to first call the {XUsb_LookupConfig()} API which returns
* the Configuration structure pointer which is passed as a parameter to the
* XUsb_CfgInitialize() API.
*
* - Configuration of the DEVICE endpoints:
*   The endpoints of the device need to be configured using the
*  	XUsb_EpConfigure() function call.
*   After configuration is complete, the endpoints need to be initialized
*   using the XUsb_EpInitializeAll() function call.
*
* <b> PHY Communication </b>
*
* As the H/W doesn't give any provision for the driver to configure the PHY, the
* driver doesn't provide any mechanism for directly programming the PHY.
*
* <b> Interrupts </b>
*
* The driver provides an interrupt handler {XUsb_IntrHandler} for handling
* the interrupt from the USB device. The users of this driver have to
* register this handler with the interrupt system and provide the callback
* functions.
* The interrupt handlers and associated callback functions for the USB device
* have to be registered by the user using the {XUsb_IntrSetHandler()} function
* and/or {XUsb_EpSetHandler()} function.
*
* {XUsb_IntrSetHandler()} function installs an asynchronous callback function
* for the general interrupts (interrupts other than the endpoint interrupts).
*
* {XUsb_EpSetHandler()} function installs the callback functions for the
* interrupts related to the endpoint events. A separate callback function has to
* be installed for each of the endpoints.
*
* <b> Virtual Memory </b>
*
* This driver supports Virtual Memory. The RTOS is responsible for calculating
* the correct device base address in Virtual Memory space.
*
* <b> Threads </b>
*
* This driver is not thread safe. Any needs for threads or thread mutual
* exclusion must be satisfied by the layer above this driver.
*
* <b> Asserts </b>
*
* Asserts are used within all Xilinx drivers to enforce constraints on argument
* values. Asserts can be turned off on a system-wide basis by defining, at
* compile time, the NDEBUG identifier. By default, asserts are turned on and it
* is recommended that users leave asserts on during development.
*
* <b> Building the driver </b>
*
* The XUsb driver is composed of several source files. This allows the user
* to build and link only those parts of the driver that are necessary.
*
* <pre>
* MODIFICATION HISTORY:
*
* Ver   Who  Date     Changes
* ----- ---- ------------------------------------------------------------------
* 1.00a hvm  02/22/07 First release.
*
* </pre>
*
 ******************************************************************************/
#ifndef XUSB_H
#define XUSB_H

#ifdef __cplusplus
extern "C" {
#endif

/***************************** Include Files *********************************/

#include "xusb_l.h"
#include "xstatus.h"

/************************** Constant Definitions *****************************/

/**
 * @name 	Endpoint Direction
 * Definitions to be used with Endpoint related function that require a
 * 'Direction' parameter.
 *
 * NOTE:
 *   The direction is always defined from the perspective of the HOST!  This
 *   means that an IN endpoint on the device is used for sending data while the
 *   OUT endpoint on the device is used for receiving data.
 * @{
 */
#define XUSB_EP_DIRECTION_IN		1	/**< Endpoint direction IN. */
#define XUSB_EP_DIRECTION_OUT		0	/**< Endpoint direction OUT. */

/* @} */

/**
 * @name Speed
 * Definitions to be used with speed.
 * @{
 */
#define XUSB_EP_HIGH_SPEED		1	/**< High Speed */
#define XUSB_EP_FULL_SPEED		0	/**< Full Speed */

/* @} */

/** @name  USB device specific global configuration constants.
 *  @{
 */
#define XUSB_MAX_ENDPOINTS		8	/**< Maximum End Points */
#define XUSB_EP_NUMBER_ZERO		0	/**< End point Zero */
#define XUSB_EP_NUMBER_ONE		1	/**< End point One */
#define XUSB_EP_NUMBER_TWO		2	/**< End point Two */
#define XUSB_EP_NUMBER_THREE		3	/**< End point Three */
#define XUSB_EP_NUMBER_FOUR		4	/**< End point Four */
#define XUSB_EP_NUMBER_FIVE		5	/**< End point Five */
#define XUSB_EP_NUMBER_SIX		6	/**< End point Six */
#define XUSB_EP_NUMBER_SEVEN		7	/**< End point Seven */
#define XUSB_DEVICEADDR_MAX		127	/**< Max device address */

/* @} */

/** @name USB device disconnect state
 * @{
 */
#define XUSB_DISCONNECTED		0x1	/**< Disconnected state */
#define XUSB_RESET			0x0	/**< Reset State */
/* @} */

/**  @name Test Modes (Set Feature)
 * @{
 */
#define TEST_J				1	/**< Chirp J Test */
#define TEST_K				2	/**< Chirp K Test */
#define TEST_SE0_NAK			3	/**< Chirp SE0 Test */
#define TEST_PKT			4	/**< Packet Test */

/* @} */

/**************************** Type Definitions *******************************/

/*****************************************************************************/
/**
 * This data type defines the callback function to be used for Endpoint
 * handlers.
 *
 * @param	CallBackRef is the Callback reference passed in by the upper
 * 		layer when setting the handler, and is passed back to the upper
 * 		layer when the handler is called.
 * @param	EpNum is the endpoint that caused the event.
 * @param	EventType is the type of the event that occurred on that
 * 		endpoint.
 */
typedef void (*XUsb_EpHandlerFunc) (void *CallBackRef, u8 EpNum, u32 EventType);

/*****************************************************************************/
/**
 * This data type defines the callback function to be used for the general
 * interrupt handler.
 *
 * @param	CallBackRef is the Callback reference passed in by the upper
 * 		layer when setting the handler, and is passed back to the upper
 *		layer when the handler is called.
 * @param	InterruptType is the type of the event that caused the
 * 		interrupt.
 */
typedef void (*XUsb_IntrHandlerFunc) (void *CallBackRef, u32 InterruptType);

/**
 * The XUsb_EpConfig structure is used to configure endpoints.
 */
typedef struct {
	int OutIn;		/**< The end point direction */
	int EpType;		/**< Bulk/interrupt/Isochronous */
	int Buffer0Count;	/**< Pkt Size for the first ping-pong 
					buffer */
	int Buffer0Ready;	/**< Status flag for first ping-pong 
					buffer */
	int Buffer1Count;	/**< Pkt Size for the second ping-pong 
					buffer */
	int Buffer1Ready;	/**< Status flag for second ping-pong 
					buffer */
	u32 Size;		/**< Maximum buffer size for this end 
					point */
	u32 RamBase;		/**< The rambase offset value in the 
					end point buffer space */
	int CurBufNum;			/**< The current ping-pong buffer to be 
					used */

	XUsb_EpHandlerFunc HandlerFunc;	/**< Call back function for this end 
						point */
	void *HandlerRef;		/**< Callback reference */

} XUsb_EpConfig;

/**
 * The XUsb_DeviceConfig structure contains the configuration information to
 * configure the USB controller for DEVICE mode. This data structure is used
 * with the XUsb_ConfigureDevice() function call.
 */
typedef struct {
	u8 NumEndpoints;	/**< Number of Endpoints */
	XUsb_EpConfig Ep[XUSB_MAX_ENDPOINTS];	/**< An array of end points */
	u8 Status;		/**< USB device Status */
	u8 CurrentConfiguration;     /**< Current state of enumeration 
				enumerated (1)/Not enumerated (0)*/
	u32 CurrentSpeed;	/**< Current Speed */

} XUsb_DeviceConfig;

/**
 * The XUsb_Config structure contains configuration information for the USB
 * controller.
 *
 * This structure only contains the basic configuration for the device. The
 * caller also needs to initialize the USB device controller with the
 * XUsb_ConfigureDevice() function call.
 */
typedef struct {
	u16 DeviceId;		/**< Unique ID of device. */
	u32 BaseAddress;	/**< Core register base address. */

} XUsb_Config;


/**
 * The XUsb driver instance data. The user is required to allocate a
 * variable of this type for every USB device in the system. A pointer
 * to a variable of this type is then passed to the driver API functions.
 */
typedef struct {
	XUsb_Config Config;	/**< Configuration structure */

	u32 IsReady;		/**< Device is initialized and ready */

	u32 USBAddress;		/**< The USB address of the device. */

	u32 EndPointOffset[XUSB_MAX_ENDPOINTS];	    /**< End point Offsets */

	/**
	 * The following structure holds the configuration for the controller.
	 * They are initialized using the XUsb_ConfigureDevice() function
	 * call.
	 */
	XUsb_DeviceConfig DeviceConfig;

	/**
	 * Callbacks and callback references
	 */
	XUsb_IntrHandlerFunc HandlerFunc;
	void *HandlerRef;

} XUsb;

/***************** Macros (Inline Functions) Definitions *********************/

/************************** Variable Definitions ******************************/

/************************** Function Prototypes ******************************/

/**
 * Setup / Initialize functions
 *
 * Implemented in the file xusb.c
 */
int XUsb_CfgInitialize(XUsb * InstancePtr, XUsb_Config * ConfigPtr,
		       u32 EffectiveAddr);

int XUsb_ConfigureDevice(XUsb * InstancePtr, XUsb_DeviceConfig * CfgPtr);

void XUsb_Start(XUsb * InstancePtr);
void XUsb_Stop(XUsb * InstancePtr);

u32 XUsb_GetFrameNum(const XUsb * InstancePtr);

int XUsb_SetDeviceAddress(XUsb * InstancePtr, u8 Address);

void XUsb_SetTestMode(XUsb * InstancePtr, u8 TestMode, u8 *BufPtr);

/*
 * Functions for managing Endpoints / Transfers
 *
 * Implemented in the file xusb_endpoint.c
 */

void XUsb_EpEnable(const XUsb * InstancePtr, u8 EpNum);
void XUsb_EpDisable(const XUsb * InstancePtr, u8 EpNum);
void XUsb_EpConfigure(XUsb * InstancePtr, u8 EpNum, XUsb_EpConfig * EpCfgPtr);

int XUsb_EpDataSend(XUsb * InstancePtr, u8 EpNum, u8 *BufferPtr, u32 BufferLen);
int XUsb_EpDataRecv(XUsb * InstancePtr, u8 EpNum, u8 *BufferPtr, u32 BufferLen);

void XUsb_EpStall(const XUsb * InstancePtr, u8 EpNum);
void XUsb_EpUnstall(const XUsb * InstancePtr, u8 EpNum);

/*
 * Interrupt handling functions
 *
 * Implemented in the file xusb_intr.c
 */

void XUsb_IntrEnable(XUsb * InstancePtr, u32 IntrMask);

void XUsb_IntrDisable(XUsb * InstancePtr, u32 IntrMask);

void XUsb_IntrHandler(void *InstancePtr);

void XUsb_IntrSetHandler(XUsb * InstancePtr, void *CallBackFunc,
			 void *CallBackRef);
void XUsb_EpSetHandler(XUsb * InstancePtr, u8 EpNum,
		       XUsb_EpHandlerFunc * CallBackFunc, void *CallBackRef);

/*
 * Static configuration helper function.
 *
 * The following function is provided for non-Linux drivers such as drivers for
 * VxWorks and Xilinx standalone systems.
 *
 * Implemented in xusb_sinit.c
 */

XUsb_Config *XUsb_LookupConfig(u16 DeviceId);

#ifdef __cplusplus
}
#endif

#endif /* XUSB_H */
