//******************************************************************************
// *
// *       XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
// *       AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
// *       SOLUTIONS FOR XILINX DEVICES. BY PROVIDING THIS DESIGN, CODE,
// *       OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
// *       APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
// *       THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
// *       AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
// *       FOR YOUR IMPLEMENTATION. XILINX EXPRESSLY DISCLAIMS ANY
// *       WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
// *       IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
// *       REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
// *       INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
// *       FOR A PARTICULAR PURPOSE.
// *
// *		Copyright(C) 2005 by xilinx, Inc. All rights reserved.              
// *
// *       (c) Copyright 2007 Xilinx Inc.
// *       All rights reserved.
//*
// ******************************************************************************/

#include "pch.h"
#include "..\intrface.h"
#ifdef XILINXDD_WMI_TRACE
#include "iorw.tmh"
#endif

///////////////////////////////////////////////////////////////////////////////////////////////////
//  xilinxddReadDispatch
//      Handled incoming read requests
//
//  Arguments:
//      IN  DeviceObject
//              Device object for our device
//
//      IN  Irp
//              The read IRP to handle
//
//  Return Value:
//      NT status code
//
NTSTATUS xilinxddReadDispatch(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    )
{
    PXILINXDD_DEVICE_EXTENSION      deviceExtension;
    PFILE_OBJECT                    fileObject;
    PIO_STACK_LOCATION              irpStack;
    NTSTATUS                        status;
    PUSBD_PIPE_INFORMATION          pipeInformation;
    PVOID                           readBuffer;
    ULONG                           readLength;

    xilinxddDebugPrint(DBG_IO, DBG_TRACE, __FUNCTION__"++. IRP %p", Irp);

    deviceExtension = (PXILINXDD_DEVICE_EXTENSION)DeviceObject->DeviceExtension;
    irpStack = IoGetCurrentIrpStackLocation(Irp);
    fileObject = irpStack->FileObject;

    status = xilinxddCheckIoLock(&deviceExtension->IoLock, Irp);
    if (!NT_SUCCESS(status) || (status == STATUS_PENDING))
    {
        xilinxddDebugPrint(DBG_IO, DBG_WARN, __FUNCTION__"--. IRP %p STATUS %x", Irp, status);

        return status;
    }

    // Get the read buffer length
    readLength = irpStack->Parameters.Read.Length;
    if (readLength == 0)
    {
        // just complete 0 length request
        status = STATUS_SUCCESS;

        Irp->IoStatus.Information = 0;
        Irp->IoStatus.Status = status;
        IoCompleteRequest(Irp, IO_NO_INCREMENT);

        xilinxddDecrementIoCount(&deviceExtension->IoLock);

        xilinxddDebugPrint(DBG_IO, DBG_WARN, __FUNCTION__"--. IRP %p, STATUS %x", Irp, status);

        goto FuncExit;
    }

    // check the pipe type and direction
    if(fileObject && fileObject->FsContext)
    {

        pipeInformation = fileObject->FsContext;

        if((UsbdPipeTypeBulk != pipeInformation->PipeType) &&
           (UsbdPipeTypeInterrupt != pipeInformation->PipeType))
        {

            xilinxddDebugPrint(DBG_IO, DBG_WARN, __FUNCTION__"--. Usbd pipe type is not bulk or interrupt");

            status = STATUS_INVALID_HANDLE;
            goto FuncExit;
        }

        // Make sure direction of pipe matches operation
        if(!(pipeInformation->EndpointAddress & 0x80))
        {
            // this is an OUT pipe not valid for read
            status = STATUS_INVALID_HANDLE;
            goto FuncExit;
        }

    } else
    {

        xilinxddDebugPrint(DBG_IO, DBG_WARN, __FUNCTION__"--. Invalid handle");

        status = STATUS_INVALID_HANDLE;
        goto FuncExit;
    }



    readBuffer = MmGetSystemAddressForMdlSafe(Irp->MdlAddress, NormalPagePriority);

    status = xilinxddPipeInIo(
                deviceExtension, 
                Irp, 
                pipeInformation
                );


    xilinxddDebugPrint(DBG_IO, DBG_TRACE, __FUNCTION__"--. IRP %p STATUS %x", Irp, status);

FuncExit:
    return status;
}

///////////////////////////////////////////////////////////////////////////////////////////////////
//  xilinxddWriteDispatch
//      Handled incoming write requests
//
//  Arguments:
//      IN  DeviceObject
//              Device object for our device
//
//      IN  Irp
//              The write IRP to handle
//
//  Return Value:
//      NT status code
//
NTSTATUS xilinxddWriteDispatch(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    )
{
    PXILINXDD_DEVICE_EXTENSION      deviceExtension;
    PFILE_OBJECT                    fileObject;
    PIO_STACK_LOCATION              irpStack;
    NTSTATUS                        status;
    PUSBD_PIPE_INFORMATION          pipeInformation;
    PVOID                           writeBuffer;
    ULONG                           writeLength;

    xilinxddDebugPrint(DBG_IO, DBG_TRACE, __FUNCTION__"++. IRP %p", Irp);

    deviceExtension = (PXILINXDD_DEVICE_EXTENSION)DeviceObject->DeviceExtension;
    irpStack = IoGetCurrentIrpStackLocation(Irp);
    fileObject = irpStack->FileObject;

    status = xilinxddCheckIoLock(&deviceExtension->IoLock, Irp);
    if (!NT_SUCCESS(status) || (status == STATUS_PENDING))
    {
        xilinxddDebugPrint(DBG_IO, DBG_WARN, __FUNCTION__"--. IRP %p STATUS %x", Irp, status);

        return status;
    }

    // Get the write buffer length
    writeLength = irpStack->Parameters.Write.Length;
    if (writeLength == 0)
    {
        // just complete 0 length request
        status = STATUS_SUCCESS;

        Irp->IoStatus.Information = 0;
        Irp->IoStatus.Status = status;
        IoCompleteRequest(Irp, IO_NO_INCREMENT);

        xilinxddDecrementIoCount(&deviceExtension->IoLock);

        xilinxddDebugPrint(DBG_IO, DBG_WARN, __FUNCTION__"--. IRP %p, STATUS %x", Irp, status);

        return status;
    }

    // check the pipe type and direction
    if(fileObject && fileObject->FsContext)
    {

        pipeInformation = fileObject->FsContext;

        if((UsbdPipeTypeBulk != pipeInformation->PipeType) &&
           (UsbdPipeTypeInterrupt != pipeInformation->PipeType))
        {

            xilinxddDebugPrint(DBG_IO, DBG_WARN, __FUNCTION__"--. Usbd pipe type is not bulk or interrupt");

            status = STATUS_INVALID_HANDLE;
            goto FuncExit;
        }

        // Make sure direction of pipe matches operation
        if(pipeInformation->EndpointAddress & 0x80)
        {
            // this is an OUT pipe not valid for read
            status = STATUS_INVALID_HANDLE;
            goto FuncExit;
        }
    } else
    {

        xilinxddDebugPrint(DBG_IO, DBG_WARN, __FUNCTION__"--. Invalid handle");

        status = STATUS_INVALID_HANDLE;
        goto FuncExit;
    }



    writeBuffer = MmGetSystemAddressForMdlSafe(Irp->MdlAddress, NormalPagePriority);

    status = xilinxddPipeOutIo(
                deviceExtension, 
                Irp, 
                pipeInformation
                );

    xilinxddDebugPrint(DBG_IO, DBG_TRACE, __FUNCTION__"--. IRP %p STATUS %x", Irp, status);

FuncExit:
    return status;
}

///////////////////////////////////////////////////////////////////////////////////////////////////
//  xilinxddDeviceIoControlDispatch
//      Handled incoming IOCTL requests
//
//  Arguments:
//      IN  DeviceObject
//              Device object for our device
//
//      IN  Irp
//              The IOCTL IRP to handle
//
//  Return Value:
//      NT status code
//
NTSTATUS xilinxddDeviceIoControlDispatch(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    )
{
    PIO_STACK_LOCATION              irpStack;
    NTSTATUS                        status;
    PXILINXDD_DEVICE_EXTENSION     deviceExtension;
    PVOID                           ioBuffer;
    ULONG                           inputBufferLength;
    ULONG                           outputBufferLength;
    ULONG                           info;

    xilinxddDebugPrint(DBG_IO, DBG_TRACE, __FUNCTION__"++. IRP %p", Irp);

    deviceExtension = (PXILINXDD_DEVICE_EXTENSION)DeviceObject->DeviceExtension;

    // Get our IRP stack location
    irpStack = IoGetCurrentIrpStackLocation(Irp);

    // Get the buffer and length
    ioBuffer           = Irp->AssociatedIrp.SystemBuffer;
    inputBufferLength  = irpStack->Parameters.DeviceIoControl.InputBufferLength;
    outputBufferLength = irpStack->Parameters.DeviceIoControl.OutputBufferLength;

    status = xilinxddCheckIoLock(&deviceExtension->IoLock, Irp);
    if (!NT_SUCCESS(status) || (status == STATUS_PENDING))
    {
        xilinxddDebugPrint(DBG_IO, DBG_WARN, __FUNCTION__"--. IRP %p STATUS %x", Irp, status);

        goto FuncExit;
    }

    switch (irpStack->Parameters.DeviceIoControl.IoControlCode) 
    {

        case IOCTL_RESET_PIPE:
            {
                PFILE_OBJECT            fileObject;
                PUSBD_PIPE_INFORMATION  pipe;
                pipe = NULL;
                fileObject = NULL;

                xilinxddDebugPrint(DBG_IO, DBG_INFO, __FUNCTION__": IOCTL_RESETPIPE");
                fileObject = irpStack->FileObject;
                if(fileObject == NULL)
                {
                    status = STATUS_INVALID_PARAMETER;
                    break;
                }

                pipe = (PUSBD_PIPE_INFORMATION) fileObject->FsContext;
                if(pipe == NULL)
                {
                    status = STATUS_INVALID_PARAMETER;
                } else
                {
                    status = Vreelindd_ResetPipe(DeviceObject, pipe);
                }
                break;
            }

        case IOCTL_GET_CONFIG:
            {
                ULONG length;

                xilinxddDebugPrint(DBG_IO, DBG_INFO, __FUNCTION__": IOCTL_GET_CONFIG_DESCRIPTOR");
                if(deviceExtension->ConfigDescriptor)
                {
                    length = deviceExtension->ConfigDescriptor->wTotalLength;
                    if(outputBufferLength >= length)
                    {
                        RtlCopyMemory(ioBuffer,
                                      deviceExtension->ConfigDescriptor,
                                      length);
                        info = length;
                        status = STATUS_SUCCESS;
                    } else
                    {
                        status = STATUS_BUFFER_TOO_SMALL;
                    }
                } else
                {
                    status = STATUS_UNSUCCESSFUL;
                }
                break;
            }

        case IOCTL_RESET_DEVICE:
            status = Vreelindd_ResetDevice(DeviceObject);
            break;

    default:
        status = STATUS_INVALID_DEVICE_REQUEST;
        break;
    }

    Irp->IoStatus.Status = status;
    Irp->IoStatus.Information = info;
    IoCompleteRequest(Irp, IO_NO_INCREMENT);
    xilinxddDecrementIoCount(&deviceExtension->IoLock);

 FuncExit:
    xilinxddDebugPrint(DBG_IO, DBG_TRACE, __FUNCTION__"--. IRP %p STATUS %x", Irp, status);

    return status;
}

NTSTATUS Vreelindd_ResetPipe(IN PDEVICE_OBJECT DeviceObject, IN PUSBD_PIPE_INFORMATION PipeInfo)
{
    PURB                            urb;
    NTSTATUS                        status;
    PXILINXDD_DEVICE_EXTENSION      deviceExtension;

    //
    // initialize variables
    //

    urb = NULL;
    deviceExtension = (PXILINXDD_DEVICE_EXTENSION) DeviceObject->DeviceExtension;


    urb = ExAllocatePool(NonPagedPool, 
                         sizeof(struct _URB_PIPE_REQUEST));

    if(urb)
    {

        urb->UrbHeader.Length = (USHORT) sizeof(struct _URB_PIPE_REQUEST);
        urb->UrbHeader.Function = URB_FUNCTION_RESET_PIPE;
        urb->UrbPipeRequest.PipeHandle = PipeInfo->PipeHandle;

        status = xilinxddSubmitUrbSynch(deviceExtension, urb);

        ExFreePool(urb);
    } else
    {

        status = STATUS_INSUFFICIENT_RESOURCES;
    }

    if(NT_SUCCESS(status))
    {

        xilinxddDebugPrint(DBG_IO, DBG_TRACE, __FUNCTION__"BulkUsb_ResetPipe - success");
        status = STATUS_SUCCESS;
    } else
    {

        xilinxddDebugPrint(DBG_IO, DBG_TRACE, __FUNCTION__"BulkUsb_ResetPipe - failed");
    }

    return(status);
}



NTSTATUS
Vreelindd_GetPortStatus(
                     IN PDEVICE_OBJECT DeviceObject,
                     IN OUT PULONG     PortStatus
                     )
{
    NTSTATUS                    status;
    KEVENT                      event;
    PIRP                        irp;
    IO_STATUS_BLOCK             ioStatus;
    PIO_STACK_LOCATION          nextStack;
    PXILINXDD_DEVICE_EXTENSION  deviceExtension;

    //
    // initialize variables
    //
    deviceExtension = (PXILINXDD_DEVICE_EXTENSION) DeviceObject->DeviceExtension;
    *PortStatus = 0;
    xilinxddDebugPrint(DBG_IO, DBG_TRACE, __FUNCTION__"--. BulkUsb_ResetDevice - begins");
    KeInitializeEvent(&event, NotificationEvent, FALSE);

    irp = IoBuildDeviceIoControlRequest(
                                       IOCTL_INTERNAL_USB_GET_PORT_STATUS,
                                       deviceExtension->LowerDeviceObject,
                                       NULL,
                                       0,
                                       NULL,
                                       0,
                                       TRUE,
                                       &event,
                                       &ioStatus);

    if(NULL == irp)
    {
        xilinxddDebugPrint(DBG_IO, DBG_TRACE, __FUNCTION__"--. memory alloc for irp failed");
        status = STATUS_INSUFFICIENT_RESOURCES;
        goto FuncExit;
    }

    nextStack = IoGetNextIrpStackLocation(irp);
    ASSERT(nextStack != NULL);
    nextStack->Parameters.Others.Argument1 = PortStatus;

    status = IoCallDriver(deviceExtension->LowerDeviceObject, irp);
    if(STATUS_PENDING == status)
    {

        KeWaitForSingleObject(&event, Executive, KernelMode, FALSE, NULL);
    } else
    {
        ioStatus.Status = status;
    }
    status = ioStatus.Status;

FuncExit:
    xilinxddDebugPrint(DBG_IO, DBG_TRACE, __FUNCTION__"--. BulkUsb_GetPortStatus - ends");
    return(status);
}

NTSTATUS
BulkUsb_ResetParentPort(
                       IN PDEVICE_OBJECT DeviceObject
                       )
{
    NTSTATUS                    status;
    KEVENT                      event;
    PIRP                        irp;
    IO_STATUS_BLOCK             ioStatus;
    PIO_STACK_LOCATION          nextStack;
    PXILINXDD_DEVICE_EXTENSION  deviceExtension;

    deviceExtension = (PXILINXDD_DEVICE_EXTENSION) DeviceObject->DeviceExtension;
    xilinxddDebugPrint(DBG_IO, DBG_TRACE, __FUNCTION__"--. BulkUsb_ResetParentPort - begins");
    KeInitializeEvent(&event, NotificationEvent, FALSE);
    irp = IoBuildDeviceIoControlRequest(
                                       IOCTL_INTERNAL_USB_RESET_PORT,
                                       deviceExtension->LowerDeviceObject,
                                       NULL,
                                       0,
                                       NULL,
                                       0,
                                       TRUE,
                                       &event,
                                       &ioStatus);

    if(NULL == irp)
    {
        xilinxddDebugPrint(DBG_IO, DBG_TRACE, __FUNCTION__"--. memory alloc for irp failed");
        status = STATUS_INSUFFICIENT_RESOURCES;
        goto FuncExit;
    }

    nextStack = IoGetNextIrpStackLocation(irp);
    ASSERT(nextStack != NULL);
    status = IoCallDriver(deviceExtension->LowerDeviceObject, irp);
    if(STATUS_PENDING == status)
    {
        KeWaitForSingleObject(&event, Executive, KernelMode, FALSE, NULL);
    } else
    {
        ioStatus.Status = status;
    }

    status = ioStatus.Status;

FuncExit:
    xilinxddDebugPrint(DBG_IO, DBG_TRACE, __FUNCTION__"--. BulkUsb_ResetParentPort - ends");
    return(status);
}


NTSTATUS Vreelindd_ResetDevice(PDEVICE_OBJECT DeviceObject)
{
    NTSTATUS status;
    ULONG    portStatus;

    xilinxddDebugPrint(DBG_IO, DBG_TRACE, __FUNCTION__"--. Vreelindd_ResetDevice - begins");

    status = Vreelindd_GetPortStatus(DeviceObject, &portStatus);

    if((NT_SUCCESS(status))                 &&
       (!(portStatus & USBD_PORT_ENABLED))    &&
       (portStatus & USBD_PORT_CONNECTED))
    {

        status = BulkUsb_ResetParentPort(DeviceObject);
    }

    xilinxddDebugPrint(DBG_IO, DBG_TRACE, __FUNCTION__"--. Vreelindd_ResetDevice - ends");

    return(status);
}

